@@ -21,15 +21,14 @@ ds_with_tuple = airds.rename({"air": (1, 2, 3)})
 
 # POM dataset
 pomds = xr.Dataset()
+# fmt: off
 pomds["sigma"] = (
-    # fmt: off
     "sigma",
     [-0.983333, -0.95    , -0.916667, -0.883333, -0.85    , -0.816667,
      -0.783333, -0.75    , -0.716667, -0.683333, -0.65    , -0.616667,
      -0.583333, -0.55    , -0.516667, -0.483333, -0.45    , -0.416667,
      -0.383333, -0.35    , -0.316667, -0.283333, -0.25    , -0.216667,
      -0.183333, -0.15    , -0.116667, -0.083333, -0.05    , -0.016667],
-    # fmt: on
     {
         "units": "sigma_level",
         "long_name": "Sigma Stretched Vertical Coordinate at Nodes",
@@ -38,6 +37,7 @@ pomds["sigma"] = (
         "formula_terms": "sigma: sigma eta: zeta depth: depth",
     }
 )
+# fmt: on
 pomds["depth"] = 175.0
 pomds["zeta"] = ("ocean_time", [-0.155356, -0.127435])
 
@@ -109,15 +109,14 @@ multiple["v2"] = (("x2", "y2"), np.ones((10, 5)) * 15)
 
 
 romsds = xr.Dataset()
+# fmt: off
 romsds["s_rho"] = (
-    # fmt: off
     "s_rho",
     [-0.983333, -0.95    , -0.916667, -0.883333, -0.85    , -0.816667,
      -0.783333, -0.75    , -0.716667, -0.683333, -0.65    , -0.616667,
      -0.583333, -0.55    , -0.516667, -0.483333, -0.45    , -0.416667,
      -0.383333, -0.35    , -0.316667, -0.283333, -0.25    , -0.216667,
      -0.183333, -0.15    , -0.116667, -0.083333, -0.05    , -0.016667],
-    # fmt: on
     {
         "long_name": "S-coordinate at RHO-points",
         "valid_min": -1.0,
@@ -125,13 +124,14 @@ romsds["s_rho"] = (
         "standard_name": "ocean_s_coordinate_g2",
         "formula_terms": "s: s_rho C: Cs_r eta: zeta depth: h depth_c: hc",
         "field": "s_rho, scalar",
-    }
+    },
 )
+# fmt: on
 romsds.coords["hc"] = 20.0
 romsds.coords["h"] = 603.9
 romsds.coords["Vtransform"] = 2.0
+# fmt: off
 romsds.coords["Cs_r"] = (
-    # fmt: off
     "s_rho",
     [-9.33010396e-01, -8.09234736e-01, -6.98779853e-01, -6.01008926e-01,
      -5.15058562e-01, -4.39938913e-01, -3.74609181e-01, -3.18031817e-01,
@@ -141,8 +141,8 @@ romsds.coords["Cs_r"] = (
      -2.53860004e-02, -1.95414261e-02, -1.46880431e-02, -1.06952600e-02,
      -7.45515186e-03, -4.87981407e-03, -2.89916971e-03, -1.45919898e-03,
      -5.20560097e-04, -5.75774004e-05],
-    # fmt: on
 )
+# fmt: on
 romsds["zeta"] = ("ocean_time", [-0.155356, -0.127435])
 romsds["temp"] = (
     ("ocean_time", "s_rho"),
@@ -228,6 +228,10 @@ mollwds = _create_mollw_ds()
 def _create_inexact_bounds():
     # Dataset that creates rotated pole curvilinear coordinates with CF bounds in
     # counterclockwise order that have precision issues.
+    # dataset created using: https://gist.github.com/larsbuntemeyer/105d83c1eb39b1462150d3fabca0b66b
+    rlon = np.array([17.935, 18.045, 18.155])
+    rlat = np.array([21.615, 21.725, 21.835])
+
     lon = np.array(
         [
             [64.21746363939087, 64.42305921561967, 64.62774455060337],
@@ -296,23 +300,70 @@ def _create_inexact_bounds():
 
     rotated = xr.Dataset(
         coords=dict(
+            rlon=xr.DataArray(
+                rlon,
+                dims="rlon",
+                attrs={
+                    "units": "degrees",
+                    "axis": "X",
+                    "standard_name": "grid_longitude",
+                },
+            ),
+            rlat=xr.DataArray(
+                rlat,
+                dims="rlat",
+                attrs={
+                    "units": "degrees",
+                    "axis": "Y",
+                    "standard_name": "grid_latitude",
+                },
+            ),
             lon=xr.DataArray(
                 lon,
-                dims=("x", "y"),
-                attrs={"units": "degrees_east", "bounds": "lon_bounds"},
+                dims=("rlon", "rlat"),
+                attrs={
+                    "units": "degrees_east",
+                    "bounds": "lon_bounds",
+                    "standard_name": "longitude",
+                },
             ),
             lat=xr.DataArray(
                 lat,
-                dims=("x", "y"),
-                attrs={"units": "degrees_north", "bounds": "lat_bounds"},
+                dims=("rlon", "rlat"),
+                attrs={
+                    "units": "degrees_north",
+                    "bounds": "lat_bounds",
+                    "standard_name": "latitude",
+                },
             ),
         ),
         data_vars=dict(
             lon_bounds=xr.DataArray(
-                lon_bounds, dims=("bounds", "x", "y"), attrs={"units": "degrees_east"}
+                lon_bounds,
+                dims=("bounds", "rlon", "rlat"),
+                attrs={"units": "degrees_east"},
             ),
             lat_bounds=xr.DataArray(
-                lat_bounds, dims=("bounds", "x", "y"), attrs={"units": "degrees_north"}
+                lat_bounds,
+                dims=("bounds", "rlon", "rlat"),
+                attrs={"units": "degrees_north"},
+            ),
+            rotated_pole=xr.DataArray(
+                np.zeros((), dtype=np.int32),
+                dims=None,
+                attrs={
+                    "grid_mapping_name": "rotated_latitude_longitude",
+                    "grid_north_pole_latitude": 39.25,
+                    "grid_north_pole_longitude": -162.0,
+                },
+            ),
+            temp=xr.DataArray(
+                np.random.rand(3, 3),
+                dims=("rlat", "rlon"),
+                attrs={
+                    "standard_name": "air_temperature",
+                    "grid_mapping": "rotated_pole",
+                },
             ),
         ),
     )
@@ -415,7 +466,7 @@ forecast = xr.decode_cf(
     )
 )
 
-
+# Same as flags_excl but easier to read
 basin = xr.DataArray(
     [1, 2, 1, 1, 2, 2, 3, 3, 3, 3],
     dims=("time",),
@@ -428,6 +479,54 @@ basin = xr.DataArray(
 )
 
 
+flag_excl = xr.DataArray(
+    np.array([1, 1, 2, 1, 2, 3, 3, 2], np.uint8),
+    dims=("time",),
+    coords={"time": [0, 1, 2, 3, 4, 5, 6, 7]},
+    attrs={
+        "flag_values": [1, 2, 3],
+        "flag_meanings": "flag_1 flag_2 flag_3",
+        "standard_name": "flag_mutual_exclusive",
+    },
+    name="flag_var",
+)
+
+
+flag_indep = xr.DataArray(
+    np.array([0, 1, 2, 3, 4, 5, 6, 7], dtype=np.uint8),
+    dims=("time",),
+    attrs={
+        "flag_masks": [1, 2, 4],
+        "flag_meanings": "flag_1 flag_2 flag_4",
+        "standard_name": "flag_independent",
+    },
+    name="flag_var",
+)
+
+flag_indep_uint16 = xr.DataArray(
+    np.array([1, 10, 100, 1000, 10000, 65535], dtype=np.uint16),
+    dims=("time",),
+    attrs={
+        "flag_masks": [2**i for i in range(16)],
+        "flag_meanings": " ".join([f"flag_{2**i}" for i in range(16)]),
+        "standard_name": "flag_independent",
+    },
+    name="flag_var",
+)
+
+flag_mix = xr.DataArray(
+    np.array([4, 8, 13, 5, 10, 14, 7, 3], np.uint8),
+    dims=("time",),
+    attrs={
+        "flag_values": [1, 2, 4, 8, 12],
+        "flag_masks": [1, 2, 12, 12, 12],
+        "flag_meanings": "flag_1 flag_2 flag_3 flag_4 flag_5",
+        "standard_name": "flag_mix",
+    },
+    name="flag_var",
+)
+
+
 ambig = xr.Dataset(
     data_vars={},
     coords={
@@ -604,3 +703,77 @@ dsg = xr.Dataset(
         "trajectory": ("trajectory", [0, 1], {"cf_role": "trajectory_id"}),
     },
 )
+
+
+sgrid_roms = xr.Dataset()
+sgrid_roms["grid"] = xr.DataArray(
+    0,
+    attrs=dict(
+        cf_role="grid_topology",
+        topology_dimension=2,
+        node_dimensions="xi_psi eta_psi",
+        face_dimensions="xi_rho: xi_psi (padding: both) eta_rho: eta_psi (padding: both)",
+        edge1_dimensions="xi_u: xi_psi eta_u: eta_psi (padding: both)",
+        edge2_dimensions="xi_v: xi_psi (padding: both) eta_v: eta_psi",
+        node_coordinates="lon_psi lat_psi",
+        face_coordinates="lon_rho lat_rho",
+        edge1_coordinates="lon_u lat_u",
+        edge2_coordinates="lon_v lat_v",
+        vertical_dimensions="s_rho: s_w (padding: none)",
+    ),
+)
+sgrid_roms["u"] = (("xi_u", "eta_u"), np.ones((2, 2)), {"grid": "grid"})
+
+sgrid_delft = xr.Dataset()
+sgrid_delft["grid"] = xr.DataArray(
+    0,
+    attrs=dict(
+        cf_role="grid_topology",
+        topology_dimension=2,
+        node_dimensions="inode jnode",
+        face_dimensions="icell: inode (padding: none) jcell: jnode (padding: none)",
+        node_coordinates="node_lon node_lat",
+    ),
+)
+
+
+sgrid_delft3 = xr.Dataset()
+sgrid_delft3["grid"] = xr.DataArray(
+    0,
+    attrs=dict(
+        cf_role="grid_topology",
+        topology_dimension=3,
+        node_dimensions="inode jnode knode",
+        volume_dimensions="iface: inode (padding: none) jface: jnode (padding: none) kface: knode (padding: none)",
+        node_coordinates="node_lon node_lat node_elevation",
+    ),
+)
+
+
+def point_dataset():
+    from shapely.geometry import MultiPoint, Point
+
+    da = xr.DataArray(
+        [
+            MultiPoint([(1.0, 2.0), (2.0, 3.0)]),
+            Point(3.0, 4.0),
+            Point(4.0, 5.0),
+            Point(3.0, 4.0),
+        ],
+        dims=("index",),
+        name="geometry",
+    )
+    ds = da.to_dataset()
+    return ds
+
+
+def encoded_point_dataset():
+    from .geometry import encode_geometries
+
+    ds = encode_geometries(point_dataset())
+    ds["data"] = (
+        "index",
+        np.arange(ds.sizes["index"]),
+        {"geometry": "geometry_container"},
+    )
+    return ds
