@@ -3,7 +3,8 @@ Started from xarray options.py
 """
 
 import copy
-from typing import Any, MutableMapping
+from collections.abc import MutableMapping
+from typing import Any
 
 from .utils import always_iterable
 
@@ -13,8 +14,9 @@ OPTIONS: MutableMapping[str, Any] = {
 }
 
 
-class set_options:
-    """Set options for cf-xarray in a controlled context.
+class set_options:  # numpydoc ignore=PR01,PR02
+    """
+    Set options for cf-xarray in a controlled context.
 
     Parameters
     ----------
@@ -36,7 +38,6 @@ class set_options:
     >>> ds = xr.Dataset({"elev": np.arange(1000)})
     >>> with cf_xarray.set_options(custom_criteria=my_custom_criteria):
     ...     xr.testing.assert_identical(ds["elev"], ds.cf["ssh"])
-    ...
 
     Or to set global options:
 
@@ -46,7 +47,7 @@ class set_options:
 
     def __init__(self, **kwargs):
         self.old = {}
-        for k, v in kwargs.items():
+        for k in kwargs:
             if k not in OPTIONS:
                 raise ValueError(
                     f"argument name {k!r} is not in the set of valid options {set(OPTIONS)!r}"
@@ -56,7 +57,7 @@ class set_options:
 
     def _apply_update(self, options_dict):
         options_dict = copy.deepcopy(options_dict)
-        for k, v in options_dict.items():
+        for k in options_dict:
             if k == "custom_criteria":
                 options_dict["custom_criteria"] = always_iterable(
                     options_dict["custom_criteria"], allowed=(tuple, list)
