@@ -2,7 +2,6 @@ import itertools
 import pickle
 import warnings
 from textwrap import dedent
-from urllib.request import urlopen
 
 import matplotlib as mpl
 import numpy as np
@@ -25,20 +24,28 @@ from ..datasets import (
     ds_no_attrs,
     ds_with_tuple,
     dsg,
+    flag_excl,
+    flag_indep,
+    flag_indep_uint16,
+    flag_mix,
     forecast,
     mollwds,
     multiple,
-    pomds,
     popds,
     romsds,
     rotds,
+    sgrid_delft,
+    sgrid_delft3,
+    sgrid_roms,
     vert,
 )
 from . import (
     raise_if_dask_computes,
     requires_cftime,
     requires_pint,
+    requires_pooch,
     requires_regex,
+    requires_rich,
     requires_scipy,
 )
 
@@ -49,6 +56,8 @@ datasets = [airds, airds.chunk({"lat": 5})]
 dataarrays = [airds.air, airds.air.chunk({"lat": 5})]
 objects = datasets + dataarrays
 
+xr.set_options(use_flox=False)
+
 
 def assert_dicts_identical(dict1, dict2):
     assert dict1.keys() == dict2.keys()
@@ -57,40 +66,43 @@ def assert_dicts_identical(dict1, dict2):
 
 
 def test_repr() -> None:
-
-    repr(ds_with_tuple.cf)
+    assert "air_temperature: [(1, 2, 3)]" in ds_with_tuple.cf.__repr__()
 
     # Dataset.
     # Stars: axes, coords, and std names
     actual = airds.cf.__repr__()
     expected = """\
     Coordinates:
-    - CF Axes: * X: ['lon']
-               * Y: ['lat']
-               * T: ['time']
-                 Z: n/a
+                 CF Axes: * X: ['lon']
+                          * Y: ['lat']
+                          * T: ['time']
+                            Z: n/a
 
-    - CF Coordinates: * longitude: ['lon']
-                      * latitude: ['lat']
-                      * time: ['time']
-                        vertical: n/a
+          CF Coordinates: * longitude: ['lon']
+                          * latitude: ['lat']
+                          * time: ['time']
+                            vertical: n/a
 
-    - Cell Measures:   area: ['cell_area']
-                       volume: n/a
+           Cell Measures:   area: ['cell_area']
+                            volume: n/a
 
-    - Standard Names: * latitude: ['lat']
-                      * longitude: ['lon']
-                      * time: ['time']
+          Standard Names: * latitude: ['lat']
+                          * longitude: ['lon']
+                          * time: ['time']
 
-    - Bounds:   n/a
+                  Bounds:   n/a
+
+           Grid Mappings:   n/a
 
     Data Variables:
-    - Cell Measures:   area, volume: n/a
+           Cell Measures:   area, volume: n/a
 
-    - Standard Names:   air_temperature: ['air']
+          Standard Names:   air_temperature: ['air']
+
+                  Bounds:   n/a
+
+           Grid Mappings:   n/a"""
 
-    - Bounds:   n/a
-    """
     assert actual == dedent(expected)
 
     # DataArray (Coordinates section same as Dataset)
@@ -98,114 +110,129 @@ def test_repr() -> None:
     actual = airds["air"].cf.__repr__()
     expected = """\
     Coordinates:
-    - CF Axes: * X: ['lon']
-               * Y: ['lat']
-               * T: ['time']
-                 Z: n/a
+                 CF Axes: * X: ['lon']
+                          * Y: ['lat']
+                          * T: ['time']
+                            Z: n/a
 
-    - CF Coordinates: * longitude: ['lon']
-                      * latitude: ['lat']
-                      * time: ['time']
-                        vertical: n/a
+          CF Coordinates: * longitude: ['lon']
+                          * latitude: ['lat']
+                          * time: ['time']
+                            vertical: n/a
 
-    - Cell Measures:   area: ['cell_area']
-                       volume: n/a
+           Cell Measures:   area: ['cell_area']
+                            volume: n/a
 
-    - Standard Names: * latitude: ['lat']
-                      * longitude: ['lon']
-                      * time: ['time']
+          Standard Names: * latitude: ['lat']
+                          * longitude: ['lon']
+                          * time: ['time']
 
-    - Bounds:   n/a
-    """
+                  Bounds:   n/a
+
+           Grid Mappings:   n/a"""
     assert actual == dedent(expected)
 
     # Empty Standard Names
     actual = popds.cf.__repr__()
     expected = """\
     Coordinates:
-    - CF Axes: * X: ['nlon']
-               * Y: ['nlat']
-                 Z, T: n/a
+                 CF Axes: * X: ['nlon']
+                          * Y: ['nlat']
+                            Z, T: n/a
 
-    - CF Coordinates:   longitude: ['TLONG', 'ULONG']
-                        latitude: ['TLAT', 'ULAT']
-                        vertical, time: n/a
+          CF Coordinates:   longitude: ['TLONG', 'ULONG']
+                            latitude: ['TLAT', 'ULAT']
+                            vertical, time: n/a
 
-    - Cell Measures:   area, volume: n/a
+           Cell Measures:   area, volume: n/a
 
-    - Standard Names:   n/a
+          Standard Names:   n/a
 
-    - Bounds:   n/a
+                  Bounds:   n/a
+
+           Grid Mappings:   n/a
 
     Data Variables:
-    - Cell Measures:   area, volume: n/a
+           Cell Measures:   area, volume: n/a
 
-    - Standard Names:   sea_water_potential_temperature: ['TEMP']
-                        sea_water_x_velocity: ['UVEL']
+          Standard Names:   sea_water_potential_temperature: ['TEMP']
+                            sea_water_x_velocity: ['UVEL']
 
-    - Bounds:   n/a
-    """
+                  Bounds:   n/a
+
+           Grid Mappings:   n/a"""
     assert actual == dedent(expected)
 
     # Flag DataArray
-    assert "CF Flag variable" in repr(basin.cf)
+    assert "Flag Variable" in repr(flag_excl.cf)
+    assert "Flag Variable" in repr(flag_indep.cf)
+    assert "Flag Variable" in repr(flag_indep_uint16.cf)
+    assert "Flag Variable" in repr(flag_mix.cf)
+    assert "Flag Variable" in repr(basin.cf)
 
     # "Temp" dataset
     actual = airds["air"]._to_temp_dataset().cf.__repr__()
     expected = """\
     Coordinates:
-    - CF Axes: * X: ['lon']
-               * Y: ['lat']
-               * T: ['time']
-                 Z: n/a
+                 CF Axes: * X: ['lon']
+                          * Y: ['lat']
+                          * T: ['time']
+                            Z: n/a
 
-    - CF Coordinates: * longitude: ['lon']
-                      * latitude: ['lat']
-                      * time: ['time']
-                        vertical: n/a
+          CF Coordinates: * longitude: ['lon']
+                          * latitude: ['lat']
+                          * time: ['time']
+                            vertical: n/a
 
-    - Cell Measures:   area: ['cell_area']
-                       volume: n/a
+           Cell Measures:   area: ['cell_area']
+                            volume: n/a
 
-    - Standard Names: * latitude: ['lat']
-                      * longitude: ['lon']
-                      * time: ['time']
+          Standard Names: * latitude: ['lat']
+                          * longitude: ['lon']
+                          * time: ['time']
 
-    - Bounds:   n/a
+                  Bounds:   n/a
+
+           Grid Mappings:   n/a
 
     Data Variables:
-    - Cell Measures:   area, volume: n/a
+           Cell Measures:   area, volume: n/a
 
-    - Standard Names:   air_temperature: [<this-array>]
+          Standard Names:   air_temperature: [<this-array>]
 
-    - Bounds:   n/a
-    """
+                  Bounds:   n/a
+
+           Grid Mappings:   n/a"""
     assert actual == dedent(expected)
 
     # CF roles
     actual = dsg.cf.__repr__()
-    expected = """
-    - CF Roles: * profile_id: ['profile']
-                * trajectory_id: ['trajectory']
+    expected = """\
+    Discrete Sampling Geometry:
+                CF Roles: * profile_id: ['profile']
+                          * trajectory_id: ['trajectory']
 
     Coordinates:
-    - CF Axes:   X, Y, Z, T: n/a
+                 CF Axes:   X, Y, Z, T: n/a
 
-    - CF Coordinates:   longitude, latitude, vertical, time: n/a
+          CF Coordinates:   longitude, latitude, vertical, time: n/a
 
-    - Cell Measures:   area, volume: n/a
+           Cell Measures:   area, volume: n/a
 
-    - Standard Names:   n/a
+          Standard Names:   n/a
 
-    - Bounds:   n/a
+                  Bounds:   n/a
+
+           Grid Mappings:   n/a
 
     Data Variables:
-    - Cell Measures:   area, volume: n/a
+           Cell Measures:   area, volume: n/a
 
-    - Standard Names:   n/a
+          Standard Names:   n/a
 
-    - Bounds:   n/a
-    """
+                  Bounds:   n/a
+
+           Grid Mappings:   n/a"""
     assert actual == dedent(expected)
 
 
@@ -262,15 +289,16 @@ def test_cell_measures() -> None:
     actual = ds.cf.__repr__()
     expected_repr = """\
     Data Variables:
-    - Cell Measures:   foo_measure: ['foo']
-                       volume: ['foo']
-                       area: n/a
+           Cell Measures:   foo_measure: ['foo']
+                            volume: ['foo']
+                            area: n/a
 
-    - Standard Names:   air_temperature: ['air']
-                        foo_std_name: ['foo']
+          Standard Names:   air_temperature: ['air']
+                            foo_std_name: ['foo']
 
-    - Bounds:   n/a
-    """
+                  Bounds:   n/a
+
+           Grid Mappings:   n/a"""
     assert actual.endswith(dedent(expected_repr))
 
 
@@ -301,13 +329,14 @@ def test_accessor_getattr_and_describe() -> None:
             "areacella",
         )
     )
-    ds_vertb = xr.decode_cf(vert, decode_coords="all")  # type: ignore
+    ds_vertb = xr.decode_cf(vert, decode_coords="all")
 
     assert ds_verta.cf.cell_measures == ds_vertb.cf.cell_measures
     assert ds_verta.o3.cf.cell_measures == ds_vertb.o3.cf.cell_measures
     assert ds_verta.cf.formula_terms == ds_vertb.cf.formula_terms
     assert ds_verta.o3.cf.formula_terms == ds_vertb.o3.cf.formula_terms
     assert ds_verta.cf.bounds == ds_vertb.cf.bounds
+    assert ds_verta.cf.grid_mapping_names == ds_vertb.cf.grid_mapping_names
     assert str(ds_verta.cf) == str(ds_vertb.cf)
 
 
@@ -424,7 +453,7 @@ def test_rename_like() -> None:
 @pytest.mark.parametrize(
     "attr, xrkwargs, cfkwargs",
     (
-        ("resample", {"time": "M"}, {"T": "M"}),
+        ("resample", {"time": "ME"}, {"T": "ME"}),
         ("rolling", {"lat": 5}, {"Y": 5}),
         ("groupby", {"group": "time"}, {"group": "T"}),
         ("groupby", {"group": "time.month"}, {"group": "T.month"}),
@@ -433,7 +462,6 @@ def test_rename_like() -> None:
     ),
 )
 def test_wrapped_classes(obj, attr, xrkwargs, cfkwargs):
-
     if attr in ("rolling", "coarsen"):
         # TODO: xarray bug, rolling and coarsen don't accept ellipsis
         args = ()
@@ -501,7 +529,6 @@ def test_pos_args_methods() -> None:
 
 
 def test_preserve_unused_keys() -> None:
-
     ds = airds.copy(deep=False)
     ds.time.attrs.clear()
     actual = ds.cf.sel(X=260, Y=40, time=airds.time[:2], method="nearest")
@@ -510,7 +537,6 @@ def test_preserve_unused_keys() -> None:
 
 
 def test_kwargs_expand_key_to_multiple_keys() -> None:
-
     actual = multiple.cf.isel(X=5, Y=3)
     expected = multiple.isel(x1=5, y1=3, x2=5, y2=3)
     assert_identical(actual, expected)
@@ -550,7 +576,6 @@ def test_args_methods(obj):
 
 
 def test_dataarray_getitem() -> None:
-
     air = airds.air.copy(deep=False)
     air.name = None
 
@@ -565,11 +590,11 @@ def test_dataarray_getitem() -> None:
 
 
 def test_dataarray_plot() -> None:
-
     obj = airds.air.copy(deep=False)
 
     rv = obj.isel(time=1).transpose("lon", "lat").cf.plot()
     assert isinstance(rv, mpl.collections.QuadMesh)
+    assert rv.axes is not None
     assert all(v > 180 for v in rv.axes.get_xlim())
     assert all(v < 200 for v in rv.axes.get_ylim())
     plt.close()
@@ -824,8 +849,12 @@ def test_add_bounds_nd_variable() -> None:
 
     # 2D
     expected = (
-        vertices_to_bounds(
-            np.arange(0, 13, 3).reshape(5, 1) + np.arange(-2, 2).reshape(1, 4)  # type: ignore
+        vertices_to_bounds(  # type: ignore[misc]
+            xr.DataArray(
+                np.arange(0, 13, 3).reshape(5, 1) + np.arange(-2, 2).reshape(1, 4),
+                dims=("x", "y"),
+            ),
+            out_dims=("bounds", "x", "y"),
         )
         .rename("z_bounds")
         .assign_coords(**ds.coords)
@@ -836,7 +865,7 @@ def test_add_bounds_nd_variable() -> None:
     # 2D rotated ds
     lon_bounds = (
         rotds.drop_vars(["lon_bounds"])
-        .assign(x=rotds["x"], y=rotds["y"])
+        .assign(x=rotds.cf["X"], y=rotds.cf["Y"])
         .cf.add_bounds(["lon"])
         .lon_bounds
     )
@@ -899,11 +928,10 @@ def test_bounds() -> None:
 
     # Dataset has bounds
     expected_repr = """\
-    - Bounds:   Y: ['lat_bounds']
-                lat: ['lat_bounds']
-                latitude: ['lat_bounds']
-    """
-    assert dedent(expected_repr) in ds.cf.__repr__()
+              Bounds:   Y: ['lat_bounds']
+                        lat: ['lat_bounds']
+                        latitude: ['lat_bounds']"""
+    assert expected_repr in ds.cf.__repr__()
 
     # DataArray does not have bounds
     expected_repr = airds.cf["air"].cf.__repr__()
@@ -949,6 +977,112 @@ def test_get_bounds_dim_name() -> None:
     assert mollwds.cf.get_bounds_dim_name("longitude") == "bounds"
     assert mollwds.cf.get_bounds_dim_name("lon") == "bounds"
 
+    # Be OK with bounds and coordinates having same attributes
+    # GH442
+    assert vert.cf.get_bounds_dim_name("longitude") == "bnds"
+    ds = vert.copy(deep=True)
+    ds.lat.attrs["standard_name"] = "longitude"
+    with pytest.raises(KeyError):
+        ds.cf.get_bounds_dim_name("longitude")
+
+
+def test_grid_mappings():
+    ds = rotds.copy(deep=False)
+
+    actual = ds.cf.grid_mapping_names
+    expected = {"rotated_latitude_longitude": ["rotated_pole"]}
+    assert actual == expected
+
+    expected = ds.rotated_pole
+    actual = ds.cf["rotated_latitude_longitude"]
+    assert_identical(actual, expected)
+
+    expected = ds.rotated_pole
+    actual = ds.cf["grid_mapping"]
+    assert_identical(actual, expected)
+
+    # Propagation
+    actual = ds.cf[["temp"]]
+    assert "rotated_pole" in actual.coords
+
+    actual = ds.cf["temp"]
+    assert "rotated_pole" in actual.coords
+
+    actual = ds.cf["temp"].cf["grid_mapping"]
+    assert_identical(actual, expected)
+
+    actual = ds.cf["temp"].coords["rotated_pole"].drop_vars("rotated_pole")
+    assert_identical(actual, expected)
+
+    actual = ds.cf["temp"].cf["rotated_latitude_longitude"]
+    assert_identical(actual, expected)
+
+    # not properly propagated if grid mapping variable not in coords
+    with pytest.raises(
+        ValueError, match="Grid Mapping variable rotated_pole not present."
+    ):
+        ds.temp.cf.grid_mapping_name
+
+    # check for https://github.com/xarray-contrib/cf-xarray/issues/448
+    expected = ds.rlon
+    actual = rotds.temp.cf["X"]
+    assert_identical(expected, actual)
+
+    # Test repr
+    expected = """\
+           Grid Mappings:   rotated_latitude_longitude: ['rotated_pole']"""
+    assert dedent(expected) in ds.cf.__repr__()
+    # assert dedent(expected) in ds.cf["temp"].cf.__repr__()
+
+    # grid_mapping_name
+    assert ds.cf["temp"].cf.grid_mapping_name == "rotated_latitude_longitude"
+
+    # what if there are really 2 grid mappins?
+    ds["temp2"] = ds.temp
+    ds["temp2"].attrs["grid_mapping"] = "rotated_pole2"
+    ds["rotated_pole2"] = ds.rotated_pole
+    expected = """\
+           Grid Mappings:   rotated_latitude_longitude: ['rotated_pole', 'rotated_pole2']"""
+    assert dedent(expected) in ds.cf.__repr__()
+
+    with pytest.raises(KeyError):
+        ds.cf["grid_mapping"]
+
+    assert "rotated_latitude_longitude" in ds.cf.keys()
+
+    with pytest.raises(KeyError):
+        ds.cf["grid_mapping"]
+    actual = ds.cf[["grid_mapping"]]
+    expected = ds[["rotated_pole", "rotated_pole2"]].reset_coords()
+    assert_identical(expected, actual)
+
+    expected = ds.rotated_pole
+    actual = ds.cf["temp"].cf["grid_mapping"]
+    assert_identical(actual, expected)
+    expected = ds.rotated_pole2
+    actual = ds.cf["temp2"].cf["grid_mapping"]
+    assert_identical(actual, expected)
+
+    # test _get_all with grid_mapping_var mapper
+    ds = ds.cf.set_coords("grid_mapping")
+    assert "rotated_pole" in ds.coords
+
+
+def test_bad_grid_mapping_attribute():
+    ds = rotds.copy(deep=False)
+    ds.temp.attrs["grid_mapping"] = "foo"
+    # warning when extracting a Datarray and grid_mapping does not exist
+    with pytest.warns(UserWarning):
+        ds.cf["temp"]
+    # warning when extracting a Dataset and grid_mapping does not exist
+    with pytest.warns(UserWarning):
+        ds.cf[["temp"]]
+    # this should probably also raise a warning (like cell_measures)
+    # with pytest.warns(UserWarning):
+    #    assert ds.cf.grid_mappings == {}
+    with pytest.warns(UserWarning):
+        ds.cf.get_associated_variable_names("temp", error=False)
+
 
 def test_docstring() -> None:
     assert "One of ('X'" in airds.cf.groupby.__doc__
@@ -959,7 +1093,7 @@ def test_docstring() -> None:
     assert "present in .indexes" in airds.cf.resample.__doc__
 
     # Make sure docs are up to date
-    get_all_doc: str = cf_xarray.accessor._get_all.__doc__  # type: ignore
+    get_all_doc: str = cf_xarray.accessor._get_all.__doc__  # type: ignore[assignment]
     all_keys = (
         cf_xarray.accessor._AXIS_NAMES
         + cf_xarray.accessor._COORD_NAMES
@@ -992,7 +1126,7 @@ def _check_unchanged(old, new):
             else:
                 assert v == n[k]
 
-    assert type(old) == type(new)
+    assert type(old) == type(new)  # noqa
     _check_attrs_equal(old.attrs, new.attrs)
 
     # Check coordinate attributes and data variable attributes
@@ -1035,8 +1169,8 @@ _VERTICAL_NAMES = _make_names(
         "nav_lev",
     ]
 )
-_X_NAMES = _make_names(["x", "nlon", "i", "ni"])
-_Y_NAMES = _make_names(["y", "nlat", "j", "nj"])
+_X_NAMES = _make_names(["x", "nlon", "i", "ni", "rlon"])
+_Y_NAMES = _make_names(["y", "nlat", "j", "nj", "rlat"])
 _Z_NAMES = _VERTICAL_NAMES + ["olevel", "level", "zlevel"]
 _LATITUDE_NAMES = _make_names(["lat", "latitude", "gphi", "nav_lat"])
 _LONGITUDE_NAMES = _make_names(["lon", "longitude", "glam", "nav_lon"])
@@ -1171,52 +1305,33 @@ def test_Z_vs_vertical_ROMS() -> None:
     )
 
 
-def test_param_vcoord_ocean_s_coord() -> None:
-    romsds.s_rho.attrs["standard_name"] = "ocean_s_coordinate_g2"
-    Zo_rho = (romsds.hc * romsds.s_rho + romsds.Cs_r * romsds.h) / (
-        romsds.hc + romsds.h
-    )
-    expected = romsds.zeta + (romsds.zeta + romsds.h) * Zo_rho
+def test_decode_vertical_coords() -> None:
+    # Fixes 'UnboundLocalError: cannot access local variable 'romsds' where it is not associated with a value'
+    from ..datasets import romsds
+
+    # needs standard names on `eta` and `depth` to derive computed standard name
+    romsds.h.attrs["standard_name"] = "sea_floor_depth_below_geopotential_datum"
+    romsds.zeta.attrs["standard_name"] = "sea_surface_height_above_geopotential_datum"
+
     romsds.cf.decode_vertical_coords(outnames={"s_rho": "z_rho"})
-    assert_allclose(
-        romsds.z_rho.reset_coords(drop=True), expected.reset_coords(drop=True)
-    )
 
-    romsds.s_rho.attrs["standard_name"] = "ocean_s_coordinate_g1"
-    Zo_rho = romsds.hc * (romsds.s_rho - romsds.Cs_r) + romsds.Cs_r * romsds.h
+    assert romsds.z_rho.shape == (2, 30)
+    assert romsds.z_rho.attrs["standard_name"] == "height_above_geopotential_datum"
 
-    expected = Zo_rho + romsds.zeta * (1 + Zo_rho / romsds.h)
-    romsds.cf.decode_vertical_coords(outnames={"s_rho": "z_rho"})
-    assert_allclose(
-        romsds.z_rho.reset_coords(drop=True), expected.reset_coords(drop=True)
-    )
+    romsds = romsds.drop_vars("z_rho")
 
-    romsds.cf.decode_vertical_coords(outnames={"s_rho": "ZZZ_rho"})
-    assert "ZZZ_rho" in romsds.coords
+    with pytest.raises(
+        AssertionError, match="if prefix is None, outnames must be provided"
+    ):
+        romsds.cf.decode_vertical_coords()
 
-    copy = romsds.copy(deep=False)
-    del copy["zeta"]
-    with pytest.raises(KeyError):
-        copy.cf.decode_vertical_coords(outnames={"s_rho": "z_rho"})
+    with pytest.warns(DeprecationWarning):
+        romsds.cf.decode_vertical_coords(prefix="z_rho")
 
-    copy = romsds.copy(deep=False)
-    copy.s_rho.attrs["formula_terms"] = "s: s_rho C: Cs_r depth: h depth_c: hc"
-    with pytest.raises(KeyError):
-        copy.cf.decode_vertical_coords(outnames={"s_rho": "z_rho"})
+    romsds_less_h = romsds.drop_vars(["h"])
 
-
-def test_param_vcoord_ocean_sigma_coordinate() -> None:
-    expected = pomds.zeta + pomds.sigma * (pomds.depth + pomds.zeta)
-    pomds.cf.decode_vertical_coords(outnames={"sigma": "z"})
-    assert_allclose(pomds.z.reset_coords(drop=True), expected.reset_coords(drop=True))
-
-    copy = pomds.copy(deep=False)
-    del copy["zeta"]
-    with pytest.raises(AssertionError):
-        copy.cf.decode_vertical_coords()
-
-    with pytest.raises(KeyError):
-        copy.cf.decode_vertical_coords(outnames={})
+    with pytest.raises(KeyError, match="Required term depth is absent in the dataset."):
+        romsds_less_h.cf.decode_vertical_coords(outnames={"s_rho": "z_rho"})
 
 
 def test_formula_terms() -> None:
@@ -1268,7 +1383,6 @@ def test_standard_name_mapper() -> None:
 @pytest.mark.parametrize("obj", objects)
 @pytest.mark.parametrize("attr", ["drop_vars", "set_coords"])
 def test_drop_vars_and_set_coords(obj, attr):
-
     # DataArray object has no attribute set_coords
     if not isinstance(obj, Dataset) and attr == "set_coords":
         return
@@ -1291,7 +1405,6 @@ def test_drop_vars_and_set_coords(obj, attr):
 
 @pytest.mark.parametrize("obj", objects)
 def test_drop_sel_and_reset_coords(obj):
-
     # Axis
     assert_identical(obj.drop_sel(lat=75), obj.cf.drop_sel(Y=75))
     # Coordinate
@@ -1308,7 +1421,6 @@ def test_drop_sel_and_reset_coords(obj):
 
 @pytest.mark.parametrize("ds", datasets)
 def test_drop_dims(ds):
-
     # Add data_var and coord to test _get_dims
     ds["lon_var"] = ds["lon"]
     ds = ds.assign_coords(lon_coord=ds["lon"])
@@ -1320,14 +1432,13 @@ def test_drop_dims(ds):
 
 @pytest.mark.parametrize("obj", objects)
 def test_rename(obj):
-
     cf_dict = {
         "air_temperature" if isinstance(obj, Dataset) else "longitude": "renamed"
     }
     xr_dict = {
-        "air"
-        if isinstance(obj, Dataset) and "air" in obj.data_vars
-        else "lon": "renamed"
+        (
+            "air" if isinstance(obj, Dataset) and "air" in obj.data_vars else "lon"
+        ): "renamed"
     }
     assert_identical(obj.rename(xr_dict), obj.cf.rename(cf_dict))
     assert_identical(obj.rename(**xr_dict), obj.cf.rename(**cf_dict))
@@ -1343,7 +1454,6 @@ def test_rename_tuple():
 
 @pytest.mark.parametrize("ds", datasets)
 def test_differentiate(ds):
-
     # Add data_var and coord to test _get_coords
     ds["lon_var"] = ds["lon"]
     ds = ds.assign_coords(lon_coord=ds["lon"])
@@ -1363,7 +1473,7 @@ def test_new_standard_name_mappers() -> None:
     )
     assert_identical(forecast.cf.chunk({"realization": 1}), forecast.chunk({"M": 1}))
     assert_identical(forecast.cf.isel({"realization": 1}), forecast.isel({"M": 1}))
-    assert_identical(forecast.cf.isel(**{"realization": 1}), forecast.isel(**{"M": 1}))  # type: ignore
+    assert_identical(forecast.cf.isel(realization=1), forecast.isel(M=1))
     assert_identical(
         forecast.cf.groupby("forecast_reference_time.month").mean(),
         forecast.groupby("S.month").mean(),
@@ -1414,15 +1524,14 @@ def test_groupby_special_ops() -> None:
     grouped = airds.groupby_bins("lat", np.arange(20, 50, 10))
 
     # __iter__
-    for (label, group), (cflabel, cfgroup) in zip(grouped, cfgrouped):
+    for (label, group), (cflabel, cfgroup) in zip(grouped, cfgrouped, strict=True):
         assert label == cflabel
         assert_identical(group, cfgroup)
 
     # arithmetic
     # TODO: Extremely buggy!
     # 1. cfgrouped - cfgrouped.mean() raises RecursionError
-    # 2. use_flox=True doesn't preserve attributes for some reason
-    with xr.set_options(keep_attrs=True):
+    with xr.set_options(keep_attrs=True, use_flox=True):
         expected = grouped - grouped.mean()
         actual = grouped - cfgrouped.mean()
     assert_identical(expected, actual)
@@ -1602,15 +1711,9 @@ def test_regex_match():
         assert_identical(ds.cf["temp"], ds["Tempblah"])
 
 
+@requires_pooch
 def test_cf_standard_name_table_version() -> None:
-
-    url = (
-        "https://raw.githubusercontent.com/cf-convention/cf-convention.github.io/"
-        "master/Data/cf-standard-names/current/src/cf-standard-name-table.xml"
-    )
-    expected_info, _, _ = parse_cf_standard_name_table(urlopen(url))
-    actual_info, _, _ = parse_cf_standard_name_table()
-    assert expected_info == actual_info
+    parse_cf_standard_name_table()
 
 
 def test_add_canonical_attributes_0_dim() -> None:
@@ -1641,7 +1744,6 @@ def test_datetime_like(reshape):
 @pytest.mark.parametrize("skip", ["units", None])
 @pytest.mark.parametrize("verbose", [True, False])
 def test_add_canonical_attributes(override, skip, verbose, capsys):
-
     ds = airds
     original = ds.copy(deep=True)
     cf_ds = ds.cf.add_canonical_attributes(
@@ -1651,11 +1753,11 @@ def test_add_canonical_attributes(override, skip, verbose, capsys):
     # Catch print
     captured = capsys.readouterr()
     if not verbose:
-        captured.out == ""
+        assert captured.out == ""
 
     # Attributes have been added
     for var in sum(ds.cf.standard_names.values(), []):
-        assert set(ds[var].attrs) < set(cf_ds[var].attrs)
+        assert set(ds[var].attrs) <= set(cf_ds[var].attrs)
 
     # Time units did not change
     assert ds["time"].attrs.get("units") is cf_ds["time"].attrs.get("units") is None
@@ -1693,47 +1795,123 @@ def test_add_canonical_attributes(override, skip, verbose, capsys):
     _check_unchanged(original, ds)
 
 
-@pytest.mark.parametrize("op", ["ge", "gt", "eq", "ne", "le", "lt"])
-def test_flag_features(op):
-    actual = getattr(basin.cf, f"__{op}__")("atlantic_ocean")
-    expected = getattr(basin, f"__{op}__")(1)
-    assert_identical(actual, expected)
+class TestFlags:
+    @pytest.mark.parametrize("op", ["ge", "gt", "eq", "ne", "le", "lt"])
+    def test_flag_rich_comp(self, op) -> None:
+        actual = getattr(basin.cf, f"__{op}__")("atlantic_ocean")
+        expected = getattr(basin, f"__{op}__")(1)
+        assert_identical(actual, expected)
 
+    def test_flag_excl(self) -> None:
+        for i in range(3):
+            name = f"flag_{i + 1}"
+            expected = (flag_excl == i + 1).rename(name)
+            actual = flag_excl.cf.flags[name]
+            assert_identical(actual, expected)
 
-def test_flag_isin() -> None:
-    actual = basin.cf.isin(["atlantic_ocean", "pacific_ocean"])
-    expected = basin.isin([1, 2])
-    assert_identical(actual, expected)
+    def test_flag_indep(self) -> None:
+        expected = [
+            [False, True, False, True, False, True, False, True],  # bit 1
+            [False, False, True, True, False, False, True, True],  # bit 2
+            [False, False, False, False, True, True, True, True],  # bit 3
+        ]
+        for i in range(3):
+            name = f"flag_{2**i}"
+            res = flag_indep.cf.flags[name]
+            np.testing.assert_equal(res.to_numpy(), expected[i])
 
+    def test_flag_indep_uint16(self) -> None:
+        expected = [
+            [True, False, False, False, False, True],  # bit     1
+            [False, True, False, False, False, True],  # bit     2
+            [False, False, True, False, False, True],  # bit     4
+            [False, True, False, True, False, True],  # bit     8
+            [False, False, False, False, True, True],  # bit    16
+            [False, False, True, True, False, True],  # bit    32
+            [False, False, True, True, False, True],  # bit    64
+            [False, False, False, True, False, True],  # bit   128
+            [False, False, False, True, True, True],  # bit   256
+            [False, False, False, True, True, True],  # bit   512
+            [False, False, False, False, True, True],  # bit  1024
+            [False, False, False, False, False, True],  # bit  2048
+            [False, False, False, False, False, True],  # bit  4096
+            [False, False, False, False, True, True],  # bit  8192
+            [False, False, False, False, False, True],  # bit 16384
+            [False, False, False, False, False, True],  # bit 32768
+        ]
+        for i in range(16):
+            name = f"flag_{2**i}"
+            res = flag_indep_uint16.cf.flags[name]
+            np.testing.assert_equal(res.to_numpy(), expected[i])
 
-def test_flag_errors() -> None:
-    with pytest.raises(ValueError):
-        basin.cf.isin(["arctic_ocean"])
+    def test_flag_mix(self) -> None:
+        expected = [
+            [False, False, True, True, False, False, True, True],  # flag 1
+            [False, False, False, False, True, True, True, True],  # flag 2
+            [True, False, False, True, False, False, True, False],  # flag 3
+            [False, True, False, False, True, False, False, False],  # flag 4
+            [False, False, True, False, False, True, False, False],  # flag 5
+        ]
+        for i in range(5):
+            name = f"flag_{i + 1}"
+            res = flag_mix.cf.flags[name]
+            np.testing.assert_equal(res.to_numpy(), expected[i])
 
-    with pytest.raises(ValueError):
-        basin.cf == "arctic_ocean"
+    @pytest.mark.parametrize(
+        "da, key", ((flag_indep, "flag_1"), (flag_mix, "flag_4"), (flag_excl, "flag_3"))
+    )
+    def test_flag_eq_ne(self, da, key) -> None:
+        assert_identical(da.cf.flags[key], (da.cf == key).rename(key))
+        assert_identical(~da.cf.flags[key], (da.cf != key).rename(key))
 
-    ds = xr.Dataset({"basin": basin})
-    with pytest.raises(ValueError):
-        ds.cf.isin(["atlantic_ocean"])
+    def test_flag_isin(self) -> None:
+        actual = flag_excl.cf.isin(["flag_1", "flag_3"])
+        expected = flag_excl.isin([1, 3])
 
-    basin.attrs.pop("flag_values")
-    with pytest.raises(ValueError):
-        basin.cf.isin(["pacific_ocean"])
+        actual = basin.cf.isin(["atlantic_ocean", "pacific_ocean"])
+        expected = basin.isin([1, 2])
+        assert_identical(actual, expected)
 
-    with pytest.raises(ValueError):
-        basin.cf == "pacific_ocean"
+        actual = flag_excl.cf.isin(["flag_ERR"])
+        assert not actual.any()
+
+    def test_flag_errors(self) -> None:
+        with pytest.raises(ValueError):
+            flag_mix.cf == "ERR"
+
+        flag_ex = flag_excl.copy()
+        flag_ex.attrs.pop("flag_values")
+        with pytest.raises(ValueError):
+            flag_ex.cf.isin(["flag_1"])
+
+        with pytest.raises(ValueError):
+            flag_ex.cf == "flag_1"
+
+        with pytest.raises(ValueError):
+            basin.cf == "arctic_ocean"
+
+        ds = xr.Dataset({"basin": basin})
+        with pytest.raises(ValueError):
+            ds.cf.isin(["atlantic_ocean"])
+
+        basin_ = basin.copy(deep=True)
+        basin_.attrs.pop("flag_values")
+        with pytest.raises(ValueError):
+            basin_.cf.isin(["pacific_ocean"])
+
+        with pytest.raises(ValueError):
+            basin_.cf == "pacific_ocean"
 
 
 def test_missing_variables() -> None:
-
     # Bounds
     ds = mollwds.copy(deep=False)
     ds = ds.drop_vars("lon_bounds")
     assert ds.cf.bounds == {"lat": ["lat_bounds"], "latitude": ["lat_bounds"]}
 
     with pytest.raises(KeyError, match=r"No results found for 'longitude'."):
-        ds.cf.get_bounds("longitude")
+        with pytest.warns(UserWarning, match="not found in object"):
+            ds.cf.get_bounds("longitude")
 
     # Cell measures
     ds = airds.copy(deep=False)
@@ -1766,6 +1944,20 @@ def test_cf_role() -> None:
     dsg.foo.cf.plot(x="trajectory_id")
 
 
+def test_grid_topology() -> None:
+    ds = xr.Dataset(
+        data_vars={},
+        coords={
+            "time": ("time", [1, 2, 3], {"standard_name": "time"}),
+            "mesh": (tuple(), 1, {"cf_role": "mesh_topology"}),
+            "grid": (tuple(), 1, {"cf_role": "grid_topology"}),
+        },
+    )
+    assert_identical(ds.cf["grid_topology"], ds.grid.reset_coords(drop=True))
+    assert_identical(ds.cf["mesh_topology"], ds.mesh.reset_coords(drop=True))
+    assert "T" in ds.cf.axes
+
+
 @requires_scipy
 def test_curvefit() -> None:
     from cf_xarray.datasets import airds
@@ -1774,16 +1966,117 @@ def test_curvefit() -> None:
         t = (time - time[0]).astype(float)
         return slope * t
 
-    actual = airds.air.cf.isel(lat=4, lon=5).curvefit(coords=("time",), func=line)
-    expected = airds.air.cf.isel(lat=4, lon=5).cf.curvefit(coords="T", func=line)
+    da = airds.air.cf.isel(lat=3, lon=3)
+    expected = da.curvefit(coords=("time",), func=line)
+    actual = da.cf.curvefit(coords="T", func=line)
     assert_identical(expected, actual)
 
     def plane(coords, slopex, slopey):
         x, y = coords
         return slopex * (x - x.mean()) + slopey * (y - y.mean())
 
-    actual = airds.air.isel(time=0).curvefit(coords=("lat", "lon"), func=plane)
-    expected = airds.air.isel(time=0).cf.curvefit(
-        coords=["latitude", "longitude"], func=plane
-    )
+    da = airds.air.cf.isel(time=0)
+    actual = da.curvefit(coords=("lat", "lon"), func=plane)
+    expected = da.cf.curvefit(coords=["latitude", "longitude"], func=plane)
     assert_identical(expected, actual)
+
+
+@requires_rich
+@pytest.mark.parametrize(
+    "obj, contains",
+    (
+        [airds.air, "Coordinates"],
+        [popds, "Data Variables"],
+        [basin, "Flag Variable"],
+        [flag_mix, "Flag Variable"],
+        [flag_indep, "Flag Variable"],
+        [flag_indep_uint16, "Flag Variable"],
+        [flag_excl, "Flag Variable"],
+        [dsg, "Discrete Sampling Geometry"],
+    ),
+)
+def test_rich_repr(obj, contains):
+    from rich.console import Console
+
+    console = Console()
+    console.begin_capture()
+    console.print(obj.cf)
+    printed = console.end_capture()
+
+    assert contains in printed
+
+
+def test_sgrid():
+    from ..sgrid import parse_axes
+
+    positions = ["psi", "u", "v", "rho"]
+    expected = {
+        "X": {f"xi_{pos}" for pos in positions},
+        "Y": {f"eta_{pos}" for pos in positions},
+        "Z": {"s_rho", "s_w"},
+    }
+    assert parse_axes(sgrid_roms) == expected
+    assert sgrid_roms.cf.axes == {"X": ["xi_u"], "Y": ["eta_u"]}
+
+    assert "grid" in sgrid_roms.cf["u"].coords
+    assert sgrid_roms.cf.get_associated_variable_names("u")["grid"] == ["grid"]
+
+    assert_identical(sgrid_roms.cf["X"], sgrid_roms.xi_u)
+    assert_identical(sgrid_roms.cf["Y"], sgrid_roms.eta_u)
+
+    with pytest.raises(KeyError):
+        sgrid_roms.u.cf["X"]
+    with pytest.raises(KeyError):
+        sgrid_roms.u.cf["Y"]
+
+    roms_ = sgrid_roms.set_coords("grid")
+    assert roms_.u.cf.axes == {"X": ["xi_u"], "Y": ["eta_u"]}
+    assert_identical(roms_.u.cf["X"], sgrid_roms.xi_u)
+    assert_identical(roms_.u.cf["Y"], sgrid_roms.eta_u)
+
+    for obj in [roms_, roms_.u]:
+        assert "xi_u" in obj.cf.__repr__()
+        assert "eta_u" in obj.cf.__repr__()
+
+    assert parse_axes(sgrid_delft) == {"X": {"icell", "inode"}, "Y": {"jcell", "jnode"}}
+    assert parse_axes(sgrid_delft3) == {
+        "X": {"iface", "inode"},
+        "Y": {"jface", "jnode"},
+        "Z": {"kface", "knode"},
+    }
+
+
+def test_ancillary_variables_extra_dim():
+    ds = xr.Dataset(
+        {
+            "x": (
+                "x",
+                range(10),
+                {"axis": "X", "ancillary_variables": "position_flag"},
+            ),
+        }
+    )
+    with pytest.warns(UserWarning):
+        assert_identical(ds.cf["X"], ds["x"])
+
+
+def test_geometry_association(geometry_ds):
+    cf_ds, _ = geometry_ds
+    actual = cf_ds.cf[["data"]]
+    for name in ["geometry_container", "x", "y", "node_count", "crd_x", "crd_y"]:
+        assert name in actual.coords
+
+    actual = cf_ds.cf["data"]
+    for name in ["geometry_container", "node_count", "crd_x", "crd_y"]:
+        assert name in actual.coords
+
+    assert cf_ds.cf.geometries == {"point": ["geometry_container"]}
+    assert_identical(cf_ds.cf["geometry"], cf_ds["geometry_container"])
+    with pytest.raises(ValueError):
+        cf_ds.cf["point"]
+
+    expected = cf_ds[["geometry_container", "node_count", "x", "y", "crd_x", "crd_y"]]
+    assert_identical(
+        cf_ds.cf[["point"]],
+        expected.set_coords(["node_count", "x", "y", "crd_x", "crd_y"]),
+    )
